/*!
 * imageDiff.js
 * author: Brent M. Dingle
 * last modified: 2015
 * Example script for simple image processing: image difference comparison
 *
 * Notice there is not a large amount of safety checks on image sizes
 * Strange things may happen if images are not same size
 *
 * Other javaScript files required for this to work:
 *       none
*/
 
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// theProgram (singleton) Object                            theProgram Object
// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
var theProgram = 
{
    // ------------------------------------------------------------------------
    // Pseudo-constants
    // ------------------------------------------------------------------------
    SOURCE_IMG_CANVAS_ID: "sourceCanvas",     // canvas id used in html
    SRC2_IMG_CANVAS_ID:   "sourceCan2",       // canvas id used in html
    OUTPUT_IMG_CANVAS_ID: "outputCanvas",     // canvas id used in html
    OUT2_IMG_CANVAS_ID:   "outputCan2",       // canvas id used in html
    
    // ------------------------------------------------------------------------
    // Variables
    // ------------------------------------------------------------------------
    width:          400,           // canvas width... may be reset
    height:         400,           // canvas height... may be reset
    
    dropArea:       null,
    imageOne:       null,
    imageTwo:       null,
    delta:          null,
    resultOne:      null,
    
    width:          0,
    height:         0,
    size:           0,

    //frames:         0,
    //scale:          0,
    //factor:         0,
    
    oneLoaded:      0,
    twoLoaded:      0,
    diffDone:       0,
    
    destCanvas:     null,
    destCTX:        null,
    dest2Can:       null,
    dest2ctx:       null,
    
    // ------------------------------------------------------------------------
    // Functions
    // ------------------------------------------------------------------------
    //                                                          Main
    // ------------------------------------------------------------------------
    Main: function() 
    {
        // Setup the listeners
        this.dropArea = document.getElementById('theDropArea');
        this.dropArea.addEventListener('dragover', this.onDragOver, false);
        this.dropArea.addEventListener('drop', this.onDropFileSelect, false);

        // Get handle on the OUTPUT destCanvas
        theProgram.destCanvas = document.getElementById(theProgram.OUTPUT_IMG_CANVAS_ID);
        theProgram.destCTX = theProgram.destCanvas.getContext('2d');
                
        // Get handle on the OUTPUT dest2Can
        theProgram.dest2Can = document.getElementById(theProgram.OUT2_IMG_CANVAS_ID);
        theProgram.dest2ctx = theProgram.dest2Can.getContext('2d');
        
    },
    

    // ------------------------------------------------------------------------
    // ------------------------------------------------------------------------
    onDragOver: function(evt) 
    {
        evt.stopPropagation();
        evt.preventDefault();
        evt.dataTransfer.dropEffect = 'copy'; // Explicitly make this is a copy
    },

    // ------------------------------------------------------------------------
    // ------------------------------------------------------------------------
    onDropFileSelect: function (evt) 
    {
        evt.stopPropagation();
        evt.preventDefault();
        var files = evt.dataTransfer.files; // files that were dragged 

        // Set flags to know that new files are being loaded
        theProgram.diffDone = 0;
        theProgram.oneLoaded = 0;
        theProgram.twoLoaded = 0;
        theProgram.width = 0;
        theProgram.height = 0;
        theProgram.size = 0;
        
        
        // If the "first" file is not an image, do nothing
        var curFile = files[0];
        // Only process image file
        if ( curFile.type.match('image.*') ) 
        {
            var imgOne = new Image;
            imgOne.onload = function() 
            {
                //theProgram.dropArea.style.display = "none"; unrem to allow only 1 drop of files
                var canvas = document.getElementById(theProgram.SOURCE_IMG_CANVAS_ID);
                var ctx = canvas.getContext('2d');
                canvas.style.display = "block";
                canvas.width = imgOne.width;  // world coordinate system size
                canvas.height = imgOne.height;
                canvas.style.width = canvas.width + "px" ;  // viewport coordinate system size (like TV screen)
                canvas.style.height = canvas.height + "px";
                //canvas.style.opacity = 0;  // check if this is needed -- makes image invisible
                
                ctx.drawImage(imgOne, 0, 0);
                //alert('the original image is drawn');   // for DEBUG
                
                // Set the source data BEFORE applying any filters!
                theProgram.imageOne  = ctx.getImageData(0, 0, imgOne.width, imgOne.height);
                
                // --- begin fade stuff
                if ((imgOne.width < theProgram.width) || (theProgram.width < 1))
                {
                    theProgram.width = imgOne.width;
                }
                if ((imgOne.height < theProgram.height)|| (theProgram.height < 1))
                {
                    theProgram.height = imgOne.height;
                }
                theProgram.size = 4 * theProgram.width * theProgram.height;
                theProgram.oneLoaded = 1;
                
                theProgram.DoTheDiff();
                
                URL.revokeObjectURL(imgOne.src);  // clear up memory... may want to hold on this if doing "real" stuff
            }
            imgOne.src = URL.createObjectURL(curFile);
            
			// If the "second" file is not an image, do nothing for it
			var twoFile = files[1];
			if ( twoFile.type.match('image.*') ) 
			{
				var imgTwo = new Image;
				imgTwo.onload = function() 
				{
					var canvas2 = document.getElementById(theProgram.SRC2_IMG_CANVAS_ID);
					var ctx2 = canvas2.getContext('2d');
					canvas2.style.display = "block";
					canvas2.width = imgTwo.width;  // world coordinate system size
					canvas2.height = imgTwo.height;
					canvas2.style.width = canvas2.width + "px" ;  // viewport coordinate system size (like TV screen)
					canvas2.style.height = canvas2.height + "px";
                    
					ctx2.drawImage(imgTwo, 0, 0);
					//alert('the original image is drawn');   // for DEBUG
					
					// Set the source data BEFORE applying any filters!
					theProgram.imageTwo  = ctx2.getImageData(0, 0, imgTwo.width, imgTwo.height);

                    // --- begin work
                    if ((imgTwo.width < theProgram.width) || (theProgram.width < 1))
                    {
                        theProgram.width = imgTwo.width;
                    }
                    if ((imgTwo.height < theProgram.height) || (theProgram.height < 1))
                    {
                        theProgram.height = imgTwo.height;
                    }
                    theProgram.size = 4 * theProgram.width * theProgram.height;
                    theProgram.twoLoaded = 1;
                    
                    theProgram.DoTheDiff();
					
					URL.revokeObjectURL(imgTwo.src);  // clear up memory... may want to hold on this if doing "real" stuff
				}
				imgTwo.src = URL.createObjectURL(twoFile);
            }
            
        } 
        // else first file type is NOT image --> so do nothing
    },

    // ------------------------------------------------------------------------
    // Will return and do nothing if both images not loaded
    // TODO: make stuff work better when images NOT the same size
    //       Must take the smaller image -- done
    //       Must index into the larger image correctly -- not done
    // ------------------------------------------------------------------------
    DoTheDiff: function()
    {
		if ( (theProgram.oneLoaded == 1) && (theProgram.twoLoaded == 1))
		{
            // shorten up amount of typing
			var width = theProgram.width;
			var height = theProgram.height;
			theProgram.imageSize = 4 * width * height;
			var len = theProgram.imageSize;

			theProgram.offset = new Array(len);
			theProgram.delta = new Array(len);
			
			theProgram.resultOne = theProgram.destCTX.createImageData(width, height);
			theProgram.resultTwo = theProgram.dest2ctx.createImageData(width, height);
			for (var i = 0; i < len; i += 1) 
			{
				theProgram.delta[i] = theProgram.imageOne.data[i] - theProgram.imageTwo.data[i];
                // results get initialized to a all white and opaque
				theProgram.resultOne.data[i] = 255;
				theProgram.resultTwo.data[i] = 255;
			}

			// make things visible and correct size
			theProgram.destCanvas.style.display = "block";
			theProgram.destCanvas.width = width;
			theProgram.destCanvas.height = height;
			theProgram.destCanvas.style.width = theProgram.destCanvas.width + "px" ;
			theProgram.destCanvas.style.height = theProgram.destCanvas.height + "px";
			// draw a black square to begin things -- mostly for debugging
			theProgram.destCTX.fillStyle = '#ffffff';
			theProgram.destCTX.fillRect(0, 0, width, height);
			
			theProgram.dest2Can.style.display = "block";
			theProgram.dest2Can.width = width;
			theProgram.dest2Can.height = height;
			theProgram.dest2Can.style.width = theProgram.dest2Can.width + "px" ;
			theProgram.dest2Can.style.height = theProgram.dest2Can.height + "px";
			// draw a black square to begin things -- mostly for debugging
			theProgram.dest2ctx.fillStyle = '#ffffff';
			theProgram.dest2ctx.fillRect(0, 0, width, height);

			theProgram.diffDone = 1;
            
            // call the display routine 
            // --- allows options for displaying the difference, 
            //     i.e. call a different function for a different display
            theProgram.DisplayDiffs();
            
		} // end both images loaded

    },

    // ------------------------------------------------------------------------
    // ------------------------------------------------------------------------
    DisplayDiffs: function()
    {
        // Manually apply an alpha effect to RGB channels to achieve crossfade
		for (var i=0; i < theProgram.imageSize; i += 4)
		{
            if (   (theProgram.delta[i] == 0)      // red
                && (theProgram.delta[i+1] == 0)    // green
                && (theProgram.delta[i+2] == 0) )  // blue
            {
                theProgram.resultOne.data[i]   = 128;
                theProgram.resultOne.data[i+1] = 128;
                theProgram.resultOne.data[i+2] = 128;
                theProgram.resultOne.data[i+3] = 30;   // semi-transparent

                theProgram.resultTwo.data[i]   = 128;
                theProgram.resultTwo.data[i+1] = 128;
                theProgram.resultTwo.data[i+2] = 128;
                theProgram.resultTwo.data[i+3] = 30;   // semi-transparent
            }
            else
            {
                theProgram.resultOne.data[i]   = theProgram.imageOne.data[i];
                theProgram.resultOne.data[i+1] = theProgram.imageOne.data[i+1];
                theProgram.resultOne.data[i+2] = theProgram.imageOne.data[i+2];

                theProgram.resultTwo.data[i]   = theProgram.imageTwo.data[i];
                theProgram.resultTwo.data[i+1] = theProgram.imageTwo.data[i+1];
                theProgram.resultTwo.data[i+2] = theProgram.imageTwo.data[i+2];
            }
		}
        
        // Draw the results to the desired canvas contexts
        theProgram.destCTX.putImageData(theProgram.resultOne, 0, 0);
        theProgram.dest2ctx.putImageData(theProgram.resultTwo, 0, 0);
    },
    
    
};  // end theProgram variable
    
// ----------------------------------------------------------------------------
//                                                          window.ONLOAD
// ----------------------------------------------------------------------------
window.onload = function()
{
    // Initialize and Start the game
    theProgram.Main();
    
};

// ----------------------------------------------------------------------------
// ----------------------------------------------------------------------------
// end of file
